/* ===========================================================================
                      HEAP SORT ON AN ARRAY
Floyd's version (1964). Sorting the array A[0] .. A[dim-1] by making a heap.
You can see how the heap evolves during sorting by setting: history=true.
leftson(A[i]) is at 2*i+1; rightson(A[i]) is at 2*i+2.
/ is integer division. For instance: 5/2=2 and 1/2=0.
=========================================================================== */
#include <iostream.h>
const int dim=5;         // dimension of the array. dim should be >= 1.
const bool history=true; // if true the list of intermediate steps is printed.
int A[dim]={24,19,25,28,34};
//----------------------------------------------------------------------------
//                      AUXILIARY FUNCTIONS
void swap(int i, int j) {int temp=A[i]; A[i]=A[j]; A[j]=temp;}

void printar(int from, int size)  // It prints A[from]..A[size-1]
{for (int i=from; i<size; i=i+1) {cout << A[i] << " ";}}

void printarasheap(int from, int size) // It prints A[from]..A[size-1]
{if (2*from+2 <= size-1)
   {cout << "("; printarasheap(2*from+1,size);
    cout << "." << A[from] << "."; printarasheap(2*from+2,size); cout << ")";}
else if (2*from+1 == size-1)
   {cout << "(" << A[from] << "-"; printarasheap(2*from+1,size); cout << ")";}
else if (from <= size-1) {cout << A[from];}
else {cout << "Error in printarasheap!";}
;}
/*----------------------------------------------------------------------------
                         PERCOLATING A[i]
width1 is width-1. The array A is: A[0]..A[width1]. For the leftson(A[i]) to
exist, we need: 2*i+1 <= (width1-1), that is, i <= (width1-1)/2
Since / is integer division, we also need width1>0 (see below)
--------------------------------------------------------------------------- */
void percolate(int i, int width1)              // width1 is width-1
{int lson, maxsonpos;
if ((i <= (width1-1)/2) && (width1>0))
  {lson=2*i+1; maxsonpos=lson;
   if ((lson < width1) && (A[lson] < A[lson+1])) {maxsonpos=lson+1;};
   if (A[i] < A[maxsonpos]) {swap(i,maxsonpos); percolate(maxsonpos,width1);};
  };}
//----------------------------------------------------------------------------
//                        MAKING THE HEAP
void makeheap()
{int upper=(dim-2)/2;
for (int i=upper; i>=0; i=i-1)
   {int lson=2*(i)+1; int maxsonpos=lson;
    if ((lson < dim-1) && (A[lson] < A[lson+1])) {maxsonpos=lson+1;};
    if (A[i] < A[maxsonpos]) {swap(i,maxsonpos);
                              percolate(maxsonpos,dim-1);};
    };}
//----------------------------------------------------------------------------
void main()
{cout << "\nThe given array A of " << dim << " elements is:\n";
printar(0,dim); cout << "\n\n";
if (dim > 1)
{makeheap();
for (int i=dim; i>=2; i=i-1) {
//----------------------------------------------------------------------------
/**/ if (history) {cout << " -> The "<<i<<"-element heap is: ";
/**/               printar(0,i); cout << "= "; printarasheap(0,i);
/**/               cout << "\n -> Max is: "<< A[0];};
//----------------------------------------------------------------------------
                          swap(0,i-1); percolate(0,i-2);
//----------------------------------------------------------------------------
/**/ if (history) {cout << ". Swapping A[0]-A["<< i-1
/**/          << "] and percolating, we get the array:\n"
/**/          << "                "
/**/          << "(in ascending order to the right of . )\n";
/**/          cout << "                ";
/**/          printar(0,i-1); cout <<". "; printar(i-1,dim);
/**/          cout <<"\n\n";};
//----------------------------------------------------------------------------
                             };};
cout << "==> Sorted Array (in ascending order):\n    ";
printar(0,dim); cout <<"\n";
}
/*output:  (for dim=5, history=true)
------------------------------------------------------------------------------
 The given array A of 5 elements is:
 24 19 25 28 34

  -> The 5-element heap is: 34 28 25 24 19 = ((24.28.19).34.25)
  -> Max is: 34. Swapping A[0]-A[4] and percolating we get the array:
                 (in ascending order to the right of . )
                 28 24 25 19 . 34

  -> The 4-element heap is: 28 24 25 19 = ((24-19).28.25)
  -> Max is: 28. Swapping 0-3 and percolating we get the array:
                 (in ascending order to the right of . )
                 25 24 19 . 28 34

  -> The 3-element heap is: 25 24 19 = (24.25.19)
  -> Max is: 25. Swapping 0-2 and percolating we get the array:
                 (in ascending order to the right of . )
                 24 19 . 25 28 34

  -> The 2-element heap is: 24 19 = (24-19)
  -> Max is: 24. Swapping 0-1 and percolating we get the array:
                 (in ascending order to the right of . )
                 19 . 24 25 28 34

 ==> Sorted Array (in ascending order):
     19 24 25 28 34
--------------------------------------------------------------------- */